@@ -4,8 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,7 +25,7 @@ int get_memo_app_info(struct MemoAppInfo *ai);
 void free_MemoList(MemoList **memo);
 int get_memos(MemoList **memo_list, int sort_order);
 int get_memos2(MemoList **memo_list, int sort_order,
- 	       int modified, int deleted, int category);
+ 	       int modified, int deleted, int privates, int category);
 int pc_memo_write(struct Memo *memo, PCRecType rt, unsigned char attrib,
 		  unsigned int *unique_id);
 
