@@ -4,8 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,35 +22,131 @@
 #include <time.h>
 #include <stdlib.h>
 #include <string.h>
+#include <pi-dlp.h>
 #include "utils.h"
 #include "log.h"
 #include "prefs.h"
+#include "password.h"
 #include "print.h"
 #include "memo.h"
 
 
 #define NUM_MEMO_CAT_ITEMS 16
 
+#define CONNECT_SIGNALS 400
+#define DISCONNECT_SIGNALS 401
+
 extern GtkTooltips *glob_tooltips;
 
 struct MemoAppInfo memo_app_info;
-int memo_category = CATEGORY_ALL;
-int clist_row_selected;
-GtkWidget *clist;
-GtkWidget *memo_text;
-GtkWidget *memo_cat_menu2;
+static int memo_category = CATEGORY_ALL;
+static int clist_row_selected;
+static GtkWidget *clist;
+static GtkWidget *memo_text;
+static GtkWidget *private_checkbox;
 /*Need one extra for the ALL category */
-GtkWidget *memo_cat_menu_item1[NUM_MEMO_CAT_ITEMS+1];
-GtkWidget *memo_cat_menu_item2[NUM_MEMO_CAT_ITEMS];
-GtkWidget *category_menu1;
-GtkWidget *scrolled_window;
-GtkWidget *pane;
+static GtkWidget *memo_cat_menu_item1[NUM_MEMO_CAT_ITEMS+1];
+static GtkWidget *memo_cat_menu_item2[NUM_MEMO_CAT_ITEMS];
+static GtkWidget *category_menu1;
+static GtkWidget *category_menu2;
+static GtkWidget *scrolled_window;
+static GtkWidget *pane;
+static struct sorted_cats sort_l[NUM_MEMO_CAT_ITEMS];
+static GtkWidget *new_record_button;
+static GtkWidget *apply_record_button;
+static GtkWidget *add_record_button;
+static int record_changed;
 
 static void update_memo_screen();
 static int memo_clear_details();
 int memo_clist_redraw();
+static void connect_changed_signals(int con_or_dis);
+static int memo_find();
 
 
+static void
+set_new_button_to(int new_state)
+{
+   jpilot_logf(LOG_DEBUG, "set_new_button_to new %d old %d\n", new_state, record_changed);
+   if (record_changed==new_state) {
+      return;
+   }
+   
+   switch (new_state) {
+    case MODIFY_FLAG:
+      gtk_widget_show(apply_record_button);
+      break;
+    case NEW_FLAG:
+      gtk_widget_show(add_record_button);
+      break;
+    case CLEAR_FLAG:
+      gtk_widget_show(new_record_button);
+      break;
+    default:
+      return;
+   }
+   switch (record_changed) {
+    case MODIFY_FLAG:
+      gtk_widget_hide(apply_record_button);
+      break;
+    case NEW_FLAG:
+      gtk_widget_hide(add_record_button);
+      break;
+    case CLEAR_FLAG:
+      gtk_widget_hide(new_record_button);
+      break;
+   }
+   record_changed=new_state;
+}
+
+static void
+cb_record_changed(GtkWidget *widget,
+		  gpointer   data)
+{
+   jpilot_logf(LOG_DEBUG, "cb_record_changed\n");
+   if (record_changed==CLEAR_FLAG) {
+      connect_changed_signals(DISCONNECT_SIGNALS);
+      set_new_button_to(MODIFY_FLAG);
+   }
+}
+
+static void connect_changed_signals(int con_or_dis)
+{
+   int i;
+   static int connected=0;
+   /* CONNECT */
+   if ((con_or_dis==CONNECT_SIGNALS) && (!connected)) {
+      connected=1;
+      
+      for (i=0; i<NUM_MEMO_CAT_ITEMS; i++) {
+	 if (memo_cat_menu_item2[i]) {
+	    gtk_signal_connect(GTK_OBJECT(memo_cat_menu_item2[i]), "toggled",
+			       GTK_SIGNAL_FUNC(cb_record_changed), NULL);
+	 }
+      }
+      gtk_signal_connect(GTK_OBJECT(memo_text), "changed",
+			 GTK_SIGNAL_FUNC(cb_record_changed), NULL);
+      gtk_signal_connect(GTK_OBJECT(private_checkbox), "toggled",
+			 GTK_SIGNAL_FUNC(cb_record_changed), NULL);
+   }
+   
+   /* DISCONNECT */
+   if ((con_or_dis==DISCONNECT_SIGNALS) && (connected)) {
+      connected=0;
+      
+      for (i=0; i<NUM_MEMO_CAT_ITEMS; i++) {
+	 if (memo_cat_menu_item2[i]) {
+	    gtk_signal_disconnect_by_func(GTK_OBJECT(memo_cat_menu_item2[i]),
+					  GTK_SIGNAL_FUNC(cb_record_changed), NULL);     
+	 }
+      }
+      gtk_signal_disconnect_by_func(GTK_OBJECT(memo_text),
+				    GTK_SIGNAL_FUNC(cb_record_changed), NULL);
+      gtk_signal_disconnect_by_func(GTK_OBJECT(private_checkbox),
+				    GTK_SIGNAL_FUNC(cb_record_changed), NULL);
+   }
+}
+
 int memo_print()
 {
    long this_many;
@@ -72,10 +167,10 @@ int memo_print()
       memo_list = &memo_list1;
    }
    if (this_many==2) {
-      get_memos2(&memo_list, SORT_ASCENDING, 2, 2, memo_category);
+      get_memos2(&memo_list, SORT_ASCENDING, 2, 2, 2, memo_category);
    }
    if (this_many==3) {
-      get_memos2(&memo_list, SORT_ASCENDING, 2, 2, CATEGORY_ALL);
+      get_memos2(&memo_list, SORT_ASCENDING, 2, 2, 2, CATEGORY_ALL);
    }
 
    print_memos(memo_list);
@@ -87,6 +182,17 @@ int memo_print()
    return 0;
 }
 
+static int find_sorted_cat(int cat)
+{
+   int i;
+   for (i=0; i< NUM_MEMO_CAT_ITEMS; i++) {
+      if (sort_l[i].cat_num==cat) {
+ 	 return i;
+      }
+   }
+   return -1;
+}
+
 void cb_delete_memo(GtkWidget *widget,
 		    gpointer   data)
 {
@@ -118,16 +224,23 @@ void cb_memo_category(GtkWidget *item, int selection)
 {
    if ((GTK_CHECK_MENU_ITEM(item))->active) {
       memo_category = selection;
-      jpilot_logf(LOG_DEBUG, "memo_category = %d\n", memo_category);
+      jpilot_logf(LOG_DEBUG, "cb_memo_category() cat=%d\n", memo_category);
       memo_clear_details();
       update_memo_screen();
+      jpilot_logf(LOG_DEBUG, "Leaving cb_memo_category()\n");
    }
 }
 
 static int memo_clear_details()
 {
    int new_cat;
+   int sorted_position;
 
+   jpilot_logf(LOG_DEBUG, "memo_clear_details()\n");
+   /* Need to disconnect these signals first */
+   set_new_button_to(NEW_FLAG);
+   connect_changed_signals(DISCONNECT_SIGNALS);
+   
    gtk_text_freeze(GTK_TEXT(memo_text));
 
    gtk_text_set_point(GTK_TEXT(memo_text), 0);
@@ -136,15 +249,24 @@ static int memo_clear_details()
 
    gtk_text_thaw(GTK_TEXT(memo_text));
    
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(private_checkbox), FALSE);
+
    if (memo_category==CATEGORY_ALL) {
       new_cat = 0;
    } else {
       new_cat = memo_category;
    }
-   gtk_check_menu_item_set_active
-     (GTK_CHECK_MENU_ITEM(memo_cat_menu_item2[new_cat]), TRUE);
-   gtk_option_menu_set_history(GTK_OPTION_MENU(memo_cat_menu2), new_cat);
+   sorted_position = find_sorted_cat(new_cat);
+   if (sorted_position<0) {
+      jpilot_logf(LOG_WARN, "Category is not legal\n");
+   } else {
+      gtk_check_menu_item_set_active
+	(GTK_CHECK_MENU_ITEM(memo_cat_menu_item2[sorted_position]), TRUE);
+      gtk_option_menu_set_history(GTK_OPTION_MENU(category_menu2), sorted_position);
+   }
 
+   set_new_button_to(CLEAR_FLAG);
+   jpilot_logf(LOG_DEBUG, "Leaving memo_clear_details()\n");
    return 0;
 }
 
@@ -163,11 +285,14 @@ int memo_get_details(struct Memo *new_memo, unsigned char *attrib)
    for (i=0; i<NUM_MEMO_CAT_ITEMS; i++) {
       if (GTK_IS_WIDGET(memo_cat_menu_item2[i])) {
 	 if (GTK_CHECK_MENU_ITEM(memo_cat_menu_item2[i])->active) {
-	    *attrib = i;
+	    *attrib = sort_l[i].cat_num;
 	    break;
 	 }
       }
    }
+   if (GTK_TOGGLE_BUTTON(private_checkbox)->active) {
+      *attrib |= dlpRecAttrSecret;
+   }
    return 0;
 }
 
@@ -185,6 +310,9 @@ static void cb_add_new_record(GtkWidget *widget,
    if (flag==CLEAR_FLAG) {
       /*Clear button was hit */
       memo_clear_details();
+      connect_changed_signals(DISCONNECT_SIGNALS);
+      set_new_button_to(NEW_FLAG);
+      gtk_widget_grab_focus(GTK_WIDGET(memo_text));
       return;
    }
    if ((flag!=NEW_FLAG) && (flag!=MODIFY_FLAG)) {
@@ -201,14 +329,19 @@ static void cb_add_new_record(GtkWidget *widget,
       }
    }
    memo_get_details(&new_memo, &attrib);
+
+   set_new_button_to(CLEAR_FLAG);
+   
    pc_memo_write(&new_memo, NEW_PC_REC, attrib, &unique_id);
    free_Memo(&new_memo);
    if (flag==MODIFY_FLAG) {
       cb_delete_memo(NULL, data);
    } else {
-      /* glob_find_id = unique_id;*/
       memo_clist_redraw();
    }
+   glob_find_id = unique_id;
+   memo_find();
+   
    return;
 }
 
@@ -221,55 +354,42 @@ static void cb_clist_selection(GtkWidget      *clist,
    struct Memo *memo;/*, new_a; */
    MyMemo *mmemo;
    int i, index, count;
-#ifdef OLD_ENTRY
-   struct Memo new_memo;
-   unsigned char attrib;
-#endif
+   int sorted_position;
    clist_row_selected=row;
 
    mmemo = gtk_clist_get_row_data(GTK_CLIST(clist), row);
 
-#ifdef OLD_ENTRY
-   if (mmemo == GINT_TO_POINTER(CLIST_NEW_ENTRY_DATA)) {
-      gtk_clist_set_text(GTK_CLIST(clist), row, 0,
-			 "Fill in details, then click here again");
-      gtk_clist_set_row_data(GTK_CLIST(clist), row,
-			     GINT_TO_POINTER(CLIST_ADDING_ENTRY_DATA));
-      memo_clear_details();
-      return;
-   }
+   set_new_button_to(CLEAR_FLAG);
+   
+   connect_changed_signals(DISCONNECT_SIGNALS);
    
-   if (mmemo == GINT_TO_POINTER(CLIST_ADDING_ENTRY_DATA)) {
-      memo_get_details(&new_memo, &attrib);
-      pc_memo_write(&new_memo, NEW_PC_REC, attrib);
-      free_Memo(&new_memo);
-      update_memo_screen();
-      return;
-   }
-#endif
-
    if (mmemo==NULL) {
       return;
    }
    memo=&(mmemo->memo);
    
    index = mmemo->attrib & 0x0F;
-   if (memo_cat_menu_item2[index]==NULL) {
+   sorted_position = find_sorted_cat(index);
+   if (memo_cat_menu_item2[sorted_position]==NULL) {
       /* Illegal category */
       jpilot_logf(LOG_DEBUG, "Category is not legal\n");
-      index = count = 0;
-   } else {
-      /* We need to count how many items down in the list this is */
-      for (i=index, count=0; i>=0; i--) {
-	 if (memo_cat_menu_item2[i]) {
-	    count++;
-	 }
-      }
-      count--;
+      index = sorted_position = 0;
    }
-   gtk_check_menu_item_set_active
-     (GTK_CHECK_MENU_ITEM(memo_cat_menu_item2[index]), TRUE);
-   gtk_option_menu_set_history(GTK_OPTION_MENU(memo_cat_menu2), count);
+   /* We need to count how many items down in the list this is */
+   for (i=sorted_position, count=0; i>=0; i--) {
+      if (memo_cat_menu_item2[i]) {
+	 count++;
+      }
+   }
+   count--;
+
+   if (sorted_position<0) {
+      jpilot_logf(LOG_WARN, "Category is not legal\n");
+   } else {
+      gtk_check_menu_item_set_active
+	(GTK_CHECK_MENU_ITEM(memo_cat_menu_item2[sorted_position]), TRUE);
+   }
+   gtk_option_menu_set_history(GTK_OPTION_MENU(category_menu2), count);
 
    gtk_text_freeze(GTK_TEXT(memo_text));
 
@@ -280,36 +400,36 @@ static void cb_clist_selection(GtkWidget      *clist,
    gtk_text_insert(GTK_TEXT(memo_text), NULL,NULL,NULL, memo->text, -1);
 
    gtk_text_thaw(GTK_TEXT(memo_text));
+
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(private_checkbox),
+				mmemo->attrib & dlpRecAttrSecret);
+     
+   connect_changed_signals(CONNECT_SIGNALS);
 }
 
 static void update_memo_screen()
 {
 #define MEMO_CLIST_CHAR_WIDTH 50
-   int num_entries, entries_shown, precount;
+   int num_entries, entries_shown, i;
+   int row_count;
+   size_t copy_max_length;
    char *last;
-   long modified, deleted;
    gchar *empty_line[] = { "" };
    GdkColor color;
    GdkColormap *colormap;
    MemoList *temp_memo;
    static MemoList *memo_list=NULL;
    char str[MEMO_CLIST_CHAR_WIDTH+2];
+   int len;
+   int show_priv;
 
+   jpilot_logf(LOG_DEBUG, "update_memo_screen()\n");
+
+   row_count=((GtkCList *)clist)->rows;
+   
    free_MemoList(&memo_list);
 
-   num_entries = get_memos(&memo_list, SORT_DESCENDING);
-   gtk_clist_clear(GTK_CLIST(clist));
-   /*gtk_text_backward_delete(GTK_TEXT(memo_text1), */
-	/*		    gtk_text_get_length(GTK_TEXT(memo_text1))); */
-   
-   /*if (memo_list==NULL) { */
-   /*   return; */
-   /*} */
-
-   /*Clear the text box to make things look nice */
-   /*gtk_text_set_point(GTK_TEXT(memo_text), 0); */
-   /*gtk_text_forward_delete(GTK_TEXT(memo_text), */
-   /*gtk_text_get_length(GTK_TEXT(memo_text))); */
+   num_entries = get_memos2(&memo_list, SORT_ASCENDING, 2, 2, 1, CATEGORY_ALL);
 
    if (memo_list==NULL) {
       gtk_tooltips_set_tip(glob_tooltips, category_menu1, _("0 records"), NULL);   
@@ -318,29 +438,18 @@ static void update_memo_screen()
 
    gtk_clist_freeze(GTK_CLIST(clist));
 
-   precount=0;
+   show_priv = show_privates(GET_PRIVATES, NULL);
+
    for (temp_memo = memo_list; temp_memo; temp_memo=temp_memo->next) {
       if ( ((temp_memo->mmemo.attrib & 0x0F) != memo_category) &&
 	  memo_category != CATEGORY_ALL) {
 	 continue;
       }
 
-      get_pref(PREF_SHOW_MODIFIED, &modified, NULL);
-      get_pref(PREF_SHOW_DELETED, &deleted, NULL);
-
-      if (temp_memo->mmemo.rt == MODIFIED_PALM_REC) {
-	 if (!modified) {
-	    num_entries--;
-	    continue;
-	 }
+      if ((show_priv != SHOW_PRIVATES) && 
+	  (temp_memo->mmemo.attrib & dlpRecAttrSecret)) {
+	 continue;
       }
-      if (temp_memo->mmemo.rt == DELETED_PALM_REC) {
-	 if (!deleted) {
-	    num_entries--;
-	    continue;
-	 }
-      }
-      precount++;
    }
 
    entries_shown=0;
@@ -350,108 +459,125 @@ static void update_memo_screen()
 	 continue;
       }
 
-      get_pref(PREF_SHOW_MODIFIED, &modified, NULL);
-      get_pref(PREF_SHOW_DELETED, &deleted, NULL);
-
-      if (temp_memo->mmemo.rt == MODIFIED_PALM_REC) {
-	 if (!modified) {
-	    num_entries--;
-	    continue;
-	 }
-      }
-      if (temp_memo->mmemo.rt == DELETED_PALM_REC) {
-	 if (!deleted) {
-	    num_entries--;
-	    continue;
-	 }
+      if ((show_priv != SHOW_PRIVATES) && 
+	  (temp_memo->mmemo.attrib & dlpRecAttrSecret)) {
+	 continue;
       }
 
-      entries_shown++;
-      gtk_clist_prepend(GTK_CLIST(clist), empty_line);
-      sprintf(str, "%d. ", precount - entries_shown + 1);
-      last = (char *)memccpy(str+strlen(str), temp_memo->mmemo.memo.text,
-			     '\n', MEMO_CLIST_CHAR_WIDTH-strlen(str));
+      if (entries_shown+1>row_count) {
+	 gtk_clist_append(GTK_CLIST(clist), empty_line);
+      }
+      sprintf(str, "%d. ", entries_shown + 1);
+
+      len=strlen(temp_memo->mmemo.memo.text);
+      /* ..memo clist does not display '/n' */
+      if ((copy_max_length = len) > MEMO_CLIST_CHAR_WIDTH) {
+	 copy_max_length = MEMO_CLIST_CHAR_WIDTH;
+      }
+      last = (char *)multibyte_safe_memccpy(str, temp_memo->mmemo.memo.text,'\n', copy_max_length);
       if (last) {
 	 *(last-1)='\0';
       } else {
-	 str[MEMO_CLIST_CHAR_WIDTH]='\0';
+	 str[copy_max_length]='\0';
       }
-      gtk_clist_set_text(GTK_CLIST(clist), 0, 0, str);
-      gtk_clist_set_row_data(GTK_CLIST(clist), 0, &(temp_memo->mmemo));
+      gtk_clist_set_text(GTK_CLIST(clist), entries_shown, 0, str);
+      gtk_clist_set_row_data(GTK_CLIST(clist), entries_shown, &(temp_memo->mmemo));
 
-      if (temp_memo->mmemo.rt == NEW_PC_REC) {
+      switch (temp_memo->mmemo.rt) {
+       case NEW_PC_REC:
 	 colormap = gtk_widget_get_colormap(clist);
 	 color.red=CLIST_NEW_RED;
 	 color.green=CLIST_NEW_GREEN;
 	 color.blue=CLIST_NEW_BLUE;
 	 gdk_color_alloc(colormap, &color);
-	 gtk_clist_set_background(GTK_CLIST(clist), 0, &color);
-      }
-      if (temp_memo->mmemo.rt == DELETED_PALM_REC) {
+	 gtk_clist_set_background(GTK_CLIST(clist), entries_shown, &color);
+	 break;
+       case DELETED_PALM_REC:
 	 colormap = gtk_widget_get_colormap(clist);
 	 color.red=CLIST_DEL_RED;
 	 color.green=CLIST_DEL_GREEN;
 	 color.blue=CLIST_DEL_BLUE;
 	 gdk_color_alloc(colormap, &color);
-	 gtk_clist_set_background(GTK_CLIST(clist), 0, &color);
-      }  
-      if (temp_memo->mmemo.rt == MODIFIED_PALM_REC) {
+	 gtk_clist_set_background(GTK_CLIST(clist), entries_shown, &color);
+	 break;
+       case MODIFIED_PALM_REC:
 	 colormap = gtk_widget_get_colormap(clist);
 	 color.red=CLIST_MOD_RED;
 	 color.green=CLIST_MOD_GREEN;
 	 color.blue=CLIST_MOD_BLUE;
 	 gdk_color_alloc(colormap, &color);
-	 gtk_clist_set_background(GTK_CLIST(clist), 0, &color);
-      }  
+	 gtk_clist_set_background(GTK_CLIST(clist), entries_shown, &color);
+	 break;
+       default:
+	 gtk_clist_set_background(GTK_CLIST(clist), entries_shown, NULL);
+      }
+      entries_shown++;
    }
 
-   jpilot_logf(LOG_DEBUG, "entries_shown=%d\n",entries_shown);
-#ifdef OLD_ENTRY
-   gtk_clist_append(GTK_CLIST(clist), empty_line);
-   gtk_clist_set_text(GTK_CLIST(clist), entries_shown, 0,
-		      "Select here to add an entry");
-   gtk_clist_set_row_data(GTK_CLIST(clist), entries_shown,
-			  GINT_TO_POINTER(CLIST_NEW_ENTRY_DATA));
-#endif
+   jpilot_logf(LOG_DEBUG, "entries_shown=%d\n", entries_shown);
+
    /*If there is an item in the list, select the first one */
    if (entries_shown>0) {
       gtk_clist_select_row(GTK_CLIST(clist), 0, 0);
       /*cb_clist_selection(clist, 0, 0, (GdkEventButton *)455, ""); */
    }
    
+   for (i=row_count-1; i>=entries_shown; i--) {
+      gtk_clist_remove(GTK_CLIST(clist), i);
+   }
+
    gtk_clist_thaw(GTK_CLIST(clist));
 
    sprintf(str, _("%d of %d records"), entries_shown, num_entries);
    gtk_tooltips_set_tip(glob_tooltips, category_menu1, str, NULL);   
+
+   connect_changed_signals(CONNECT_SIGNALS);
+
+   jpilot_logf(LOG_DEBUG, "Leaving update_memo_screen()\n");
 }
 
 /*todo combine these 2 functions */
-static int make_category_menu1(GtkWidget **category_menu)
+static int make_category_menu(GtkWidget **category_menu,
+			      int include_all)
 {
    GtkWidget *menu;
    GSList    *group;
    int i;
+   int offset;
+   GtkWidget **memo_cat_menu_item;
 
+   if (include_all) {
+      memo_cat_menu_item = memo_cat_menu_item1;
+   } else {
+      memo_cat_menu_item = memo_cat_menu_item2;
+   }
+   
    *category_menu = gtk_option_menu_new();
    
    menu = gtk_menu_new();
    group = NULL;
    
-   memo_cat_menu_item1[0] = gtk_radio_menu_item_new_with_label(group, _("All"));
-   gtk_signal_connect(GTK_OBJECT(memo_cat_menu_item1[0]), "activate",
-		      cb_memo_category, GINT_TO_POINTER(CATEGORY_ALL));
-   group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(memo_cat_menu_item1[0]));
-   gtk_menu_append(GTK_MENU(menu), memo_cat_menu_item1[0]);
-   gtk_widget_show(memo_cat_menu_item1[0]);
+   offset=0;
+   if (include_all) {
+      memo_cat_menu_item[0] = gtk_radio_menu_item_new_with_label(group, _("All"));
+      gtk_signal_connect(GTK_OBJECT(memo_cat_menu_item[0]), "activate",
+			 cb_memo_category, GINT_TO_POINTER(CATEGORY_ALL));
+      group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(memo_cat_menu_item[0]));
+      gtk_menu_append(GTK_MENU(menu), memo_cat_menu_item[0]);
+      gtk_widget_show(memo_cat_menu_item[0]);
+      offset=1;
+   }
    for (i=0; i<NUM_MEMO_CAT_ITEMS; i++) {
-      if (memo_app_info.category.name[i][0]) {
-	 memo_cat_menu_item1[i+1] = gtk_radio_menu_item_new_with_label(
-		     group, memo_app_info.category.name[i]);
-	 gtk_signal_connect(GTK_OBJECT(memo_cat_menu_item1[i+1]), "activate",
-			    cb_memo_category, GINT_TO_POINTER(i));
-	 group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(memo_cat_menu_item1[i+1]));
-	 gtk_menu_append(GTK_MENU(menu), memo_cat_menu_item1[i+1]);
-	 gtk_widget_show(memo_cat_menu_item1[i+1]);
+      if (sort_l[i].Pcat[0]) {
+	 memo_cat_menu_item[i+offset] = gtk_radio_menu_item_new_with_label(
+	    group, sort_l[i].Pcat);
+	 if (include_all) {
+	    gtk_signal_connect(GTK_OBJECT(memo_cat_menu_item[i+offset]), "activate",
+	       cb_memo_category, GINT_TO_POINTER(sort_l[i].cat_num));
+	 }
+	 group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(memo_cat_menu_item[i+offset]));
+	 gtk_menu_append(GTK_MENU(menu), memo_cat_menu_item[i+offset]);
+	 gtk_widget_show(memo_cat_menu_item[i+offset]);
       }
    }
 
@@ -460,34 +586,6 @@ static int make_category_menu1(GtkWidget **category_menu)
    return 0;
 }
 
-static int make_category_menu2()
-{
-   int i;
-   
-   GtkWidget *menu;
-   GSList    *group;
-
-   memo_cat_menu2 = gtk_option_menu_new();
-   
-   menu = gtk_menu_new();
-   group = NULL;
-   
-   for (i=0; i<NUM_MEMO_CAT_ITEMS; i++) {
-      if (memo_app_info.category.name[i][0]) {
-	 memo_cat_menu_item2[i] = gtk_radio_menu_item_new_with_label
-	   (group, memo_app_info.category.name[i]);
-	 group = gtk_radio_menu_item_group
-	   (GTK_RADIO_MENU_ITEM(memo_cat_menu_item2[i]));
-	 gtk_menu_append(GTK_MENU(menu), memo_cat_menu_item2[i]);
-	 gtk_widget_show(memo_cat_menu_item2[i]);
-      } else {
-	 memo_cat_menu_item2[i] = NULL;
-      }
-   }
-   gtk_option_menu_set_menu(GTK_OPTION_MENU(memo_cat_menu2), menu);
-   return 0;
-}
-
 static int memo_find()
 {
    int r, found_at, total_count;
@@ -502,53 +600,26 @@ static int memo_find()
 	    total_count = 1;
 	 }
 	 gtk_clist_select_row(GTK_CLIST(clist), found_at, 0);
-	 move_scrolled_window_hack(scrolled_window,
-				   (float)found_at/(float)total_count);
+	 if (!gtk_clist_row_is_visible(GTK_CLIST(clist), found_at)) {
+	    move_scrolled_window_hack(scrolled_window,
+				      (float)found_at/(float)total_count);
+	 }
       }
       glob_find_id = 0;
    }
    return 0;
 }
 
-static int memo_goto_line(int line_num, gfloat percentage)
-{
-   int total_count;
-
-   clist_count(clist, &total_count);
-
-   /*avoid dividing by zero */
-   if (total_count == 0) {
-      total_count = 1;
-   }
-   gtk_clist_select_row(GTK_CLIST(clist), line_num, 0);
-   move_scrolled_window_hack(scrolled_window, percentage);
-
-   return 0;
-}
-
 /* This redraws the clist and goes back to the same line number */
 int memo_clist_redraw()
 {
    int line_num;
-   GtkScrollbar *sb;
-   gfloat upper, lower, value, step, percentage;
    
    line_num = clist_row_selected;
 
-   sb = GTK_SCROLLBAR(GTK_SCROLLED_WINDOW(scrolled_window)->vscrollbar);
-   upper = GTK_ADJUSTMENT(sb->range.adjustment)->upper;
-   lower = GTK_ADJUSTMENT(sb->range.adjustment)->lower;
-   value = GTK_ADJUSTMENT(sb->range.adjustment)->value;
-   step = GTK_ADJUSTMENT(sb->range.adjustment)->step_increment;
-   if (upper - lower + step) {
-      percentage = value/(upper - lower + step);
-   } else {
-      percentage = 0;
-   }
-
    update_memo_screen();
 
-   memo_goto_line(line_num, percentage);
+   gtk_clist_select_row(GTK_CLIST(clist), line_num, 0);
 
    return 0;
 }
@@ -563,18 +634,23 @@ int memo_refresh()
    if (memo_category==CATEGORY_ALL) {
       index=0;
    } else {
-      index=memo_category+1;
+      index=find_sorted_cat(memo_category)+1;
    }
    update_memo_screen();
-   gtk_option_menu_set_history(GTK_OPTION_MENU(category_menu1), index);
-   gtk_check_menu_item_set_active
-     (GTK_CHECK_MENU_ITEM(memo_cat_menu_item1[index]), TRUE);
+   if (index<0) {
+      jpilot_logf(LOG_WARN, "Category not legal\n");
+   } else {	 
+      gtk_option_menu_set_history(GTK_OPTION_MENU(category_menu1), index);
+      gtk_check_menu_item_set_active
+	(GTK_CHECK_MENU_ITEM(memo_cat_menu_item1[index]), TRUE);
+   }
    memo_find();   
    return 0;
 }
 
 int memo_gui_cleanup()
 {
+   connect_changed_signals(DISCONNECT_SIGNALS);
    set_pref(PREF_MEMO_PANE, GTK_PANED(pane)->handle_xpos);
    return 0;
 }
@@ -586,6 +662,7 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
 {
    extern GtkWidget *glob_date_label;
    extern int glob_date_timer_tag;
+   int i;
    GtkWidget *vbox1, *vbox2, *hbox_temp;
    GtkWidget *separator;
    GtkWidget *button;
@@ -594,10 +671,28 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    const char *svalue;
    
    clist_row_selected=0;
+   record_changed=CLEAR_FLAG;
+
+   /* Do some initialization */
+   for (i=0; i<NUM_MEMO_CAT_ITEMS; i++) {
+      memo_cat_menu_item2[i] = NULL;
+   }
 
    get_memo_app_info(&memo_app_info);
 
-   if (memo_app_info.category.name[memo_category][0]=='\0') {
+   for (i=0; i<NUM_MEMO_CAT_ITEMS; i++) {
+      sort_l[i].Pcat = memo_app_info.category.name[i];
+      sort_l[i].cat_num = i;
+   }
+   qsort(sort_l, NUM_MEMO_CAT_ITEMS, sizeof(struct sorted_cats), cat_compare);
+
+#ifdef JPILOT_DEBUG
+   for (i=0; i<NUM_MEMO_CAT_ITEMS; i++) {
+      printf("cat %d %s\n", sort_l[i].cat_num, sort_l[i].Pcat);
+   }
+#endif
+
+   if ((memo_category != CATEGORY_ALL) && (memo_app_info.category.name[memo_category][0]=='\0')) {
       memo_category=CATEGORY_ALL;
    }
 
@@ -612,27 +707,17 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
 
    gtk_paned_pack1(GTK_PANED(pane), vbox1, TRUE, FALSE);
    gtk_paned_pack2(GTK_PANED(pane), vbox2, TRUE, FALSE);
-   gtk_widget_show(pane);
 
    /* gtk_widget_set_usize(GTK_WIDGET(vbox1), 210, 0); */
 
    /* Add buttons in left vbox */
-   button = gtk_button_new_with_label(_("Delete"));
-   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		      GTK_SIGNAL_FUNC(cb_delete_memo),
-		      GINT_TO_POINTER(DELETE_FLAG));
-   gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
-   gtk_widget_show(button);
-   
    /*Separator */
    separator = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(vbox1), separator, FALSE, FALSE, 5);
-   gtk_widget_show(separator);
 
    /*Make the Today is: label */
    glob_date_label = gtk_label_new(" ");
    gtk_box_pack_start(GTK_BOX(vbox1), glob_date_label, FALSE, FALSE, 0);
-   gtk_widget_show(glob_date_label);
    timeout_date(NULL);
    glob_date_timer_tag = gtk_timeout_add(CLOCK_TICK, timeout_date, NULL);
 
@@ -640,13 +725,11 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    /*Separator */
    separator = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(vbox1), separator, FALSE, FALSE, 5);
-   gtk_widget_show(separator);
 
 
    /*Put the left-hand category menu up */
-   make_category_menu1(&category_menu1);   
+   make_category_menu(&category_menu1, TRUE);   
    gtk_box_pack_start(GTK_BOX(vbox1), category_menu1, FALSE, FALSE, 0);
-   gtk_widget_show(category_menu1);
 
 
    /*Put the memo list window up */
@@ -656,7 +739,6 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start(GTK_BOX(vbox1), scrolled_window, TRUE, TRUE, 0);
-   gtk_widget_show(scrolled_window);
 
    clist = gtk_clist_new(1);
    gtk_signal_connect(GTK_OBJECT(clist), "select_row",
@@ -671,57 +753,70 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    /*   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW */
    /*					 (scrolled_window), clist); */
    gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(clist));
-   gtk_widget_show(clist);
-   
-   
    
    /* */
    /* The right hand part of the main window follows: */
    /* */
    hbox_temp = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, FALSE, FALSE, 0);
-   gtk_widget_show(hbox_temp);
 
    
    /*Add record modification buttons on right side */
-   button = gtk_button_new_with_label(_("Add It"));
+   button = gtk_button_new_with_label(_("Delete"));
+   gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		      GTK_SIGNAL_FUNC(cb_delete_memo),
+		      GINT_TO_POINTER(DELETE_FLAG));
+   gtk_box_pack_start(GTK_BOX(hbox_temp), button, TRUE, TRUE, 0);
+   
+   button = gtk_button_new_with_label(_("Copy"));
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_add_new_record), 
 		      GINT_TO_POINTER(NEW_FLAG));
    gtk_box_pack_start(GTK_BOX(hbox_temp), button, TRUE, TRUE, 0);
-   gtk_widget_show(button);
    
-   button = gtk_button_new_with_label(_("Apply Changes"));
-   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		      GTK_SIGNAL_FUNC(cb_add_new_record), 
-		      GINT_TO_POINTER(MODIFY_FLAG));
-   gtk_box_pack_start(GTK_BOX(hbox_temp), button, TRUE, TRUE, 0);
-   gtk_widget_show(button);
-   
-   button = gtk_button_new_with_label(_("New"));
-   gtk_signal_connect(GTK_OBJECT(button), "clicked",
+   new_record_button = gtk_button_new_with_label(_("New Record"));
+   gtk_signal_connect(GTK_OBJECT(new_record_button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_add_new_record), 
 		      GINT_TO_POINTER(CLEAR_FLAG));
-   gtk_box_pack_start(GTK_BOX(hbox_temp), button, TRUE, TRUE, 0);
-   gtk_widget_show(button);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), new_record_button, TRUE, TRUE, 0);
 
+   add_record_button = gtk_button_new_with_label(_("Add Record"));
+   gtk_signal_connect(GTK_OBJECT(add_record_button), "clicked",
+		      GTK_SIGNAL_FUNC(cb_add_new_record), 
+		      GINT_TO_POINTER(NEW_FLAG));
+   gtk_box_pack_start(GTK_BOX(hbox_temp), add_record_button, TRUE, TRUE, 0);
+   gtk_widget_set_name(GTK_WIDGET(GTK_LABEL(GTK_BIN(add_record_button)->child)),
+		       "label_high");
+
+   apply_record_button = gtk_button_new_with_label(_("Apply Changes"));
+   gtk_signal_connect(GTK_OBJECT(apply_record_button), "clicked",
+		      GTK_SIGNAL_FUNC(cb_add_new_record), 
+		      GINT_TO_POINTER(MODIFY_FLAG));
+   gtk_box_pack_start(GTK_BOX(hbox_temp), apply_record_button, TRUE, TRUE, 0);
+   gtk_widget_set_name(GTK_WIDGET(GTK_LABEL(GTK_BIN(apply_record_button)->child)),
+		       "label_high");
+   
 
    /*Separator */
    separator = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(vbox2), separator, FALSE, FALSE, 5);
-   gtk_widget_show(separator);
+
+
+   /*Private check box */
+   hbox_temp = gtk_hbox_new(FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, FALSE, FALSE, 0);
+   private_checkbox = gtk_check_button_new_with_label(_("Private"));
+   gtk_box_pack_end(GTK_BOX(hbox_temp), private_checkbox, FALSE, FALSE, 0);
 
 
    /*Put the right-hand category menu up */
-   make_category_menu2();   
-   gtk_box_pack_start(GTK_BOX (vbox2), memo_cat_menu2, FALSE, FALSE, 0);
-   gtk_widget_show(memo_cat_menu2);
+   make_category_menu(&category_menu2, FALSE);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), category_menu2, TRUE, TRUE, 0);
    
    
    /*The Description text box on the right side */
    hbox_temp = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, FALSE, FALSE, 0);
-   gtk_widget_show(hbox_temp);
 
 
    hbox_temp = gtk_hbox_new (FALSE, 0);
@@ -733,14 +828,12 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    vscrollbar = gtk_vscrollbar_new(GTK_TEXT(memo_text)->vadj);
    gtk_box_pack_start(GTK_BOX(hbox_temp), memo_text, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_temp), vscrollbar, FALSE, FALSE, 0);
-   gtk_widget_show(memo_text);
-   gtk_widget_show(vscrollbar);   
-   gtk_widget_show(hbox_temp);
 
-   gtk_widget_show(vbox1);
-   gtk_widget_show(vbox2);
-   gtk_widget_show(vbox);
-   gtk_widget_show(hbox);
+   gtk_widget_show_all(vbox);
+   gtk_widget_show_all(hbox);
+
+   gtk_widget_hide(add_record_button);
+   gtk_widget_hide(apply_record_button);
 
    memo_refresh();
    memo_find();
