@@ -4,8 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,11 +28,16 @@
 #include <pi-datebook.h>
 #include "config.h"
 
+
+extern int datebook_category;
+
+
 static GtkWidget *window=NULL;
 static GtkWidget *glob_month_vbox;
 static GtkWidget *glob_hbox_month_row[6];
 static GtkWidget *glob_month_texts[31];
 static GtkWidget *glob_month_month_label;
+static GtkWidget *big_text;
 static struct tm glob_month_date;
 
 
@@ -75,6 +79,72 @@ cb_month_move(GtkWidget *widget,
    display_months_appts(&glob_month_date, glob_month_texts);
 }
 
+static void
+cb_enter_notify(GtkWidget *widget, GdkEvent *event, gpointer data)
+{
+   AppointmentList *a_list;
+   AppointmentList *temp_al;
+   struct tm date;
+   char desc[400];
+   char datef[20];
+   static int prev_day=-1;
+#ifdef USE_DB3
+   int ret;
+   int category;
+   int cat_bit;
+   long use_db3_tags;
+#endif
+
+   if (prev_day==GPOINTER_TO_INT(data)+1) {
+      return;
+   }
+   prev_day = GPOINTER_TO_INT(data)+1;
+   
+   a_list = NULL;
+   
+   memcpy(&date, &glob_month_date, sizeof(struct tm));
+   date.tm_mday=GPOINTER_TO_INT(data)+1;
+   mktime(&date);
+
+   /* Get all of the appointments */
+   get_days_appointments2(&a_list, &date, 2, 2, 2);
+
+   gtk_text_set_point(GTK_TEXT(big_text),
+		      gtk_text_get_length(GTK_TEXT(big_text)));
+   gtk_text_backward_delete(GTK_TEXT(big_text),
+			    gtk_text_get_length(GTK_TEXT(big_text)));
+
+   for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
+#ifdef USE_DB3
+      get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
+      if (use_db3_tags) {
+	 ret = db3_is_float(&(temp_al->ma.a), &category);
+	 jpilot_logf(LOG_DEBUG, "category = 0x%x\n", category);
+	 cat_bit=1<<category;
+	 if (!(cat_bit & datebook_category)) {
+	    jpilot_logf(LOG_DEBUG, "skipping rec not in this category\n");
+	    continue;
+	 }
+      }
+#endif
+      if (temp_al->ma.a.event) {
+	 desc[0]='\0';
+      } else {
+	 get_pref_time_no_secs(datef);
+	 strftime(desc, 20, datef, &(temp_al->ma.a.begin));
+      }
+      strcat(desc, " ");
+      if (temp_al->ma.a.description) {
+	 strncat(desc, temp_al->ma.a.description, 300);
+	 desc[300]='\0';
+      }
+      remove_cr_lfs(desc);
+      strcat(desc, "\n");
+      gtk_text_insert(GTK_TEXT(big_text), NULL, NULL, NULL, desc, -1);
+      }
+   free_AppointmentList(&a_list);
+}
+
 void create_month_boxes()
 {
    int i;
@@ -90,7 +160,10 @@ void destroy_month_boxes()
    int i;
    
    for (i=0; i<6; i++) {
-      gtk_widget_destroy(glob_hbox_month_row[i]);
+      if (glob_hbox_month_row[i]) {
+	 gtk_widget_destroy(glob_hbox_month_row[i]);
+	 glob_hbox_month_row[i]=NULL;
+      }
    }
 }
 
@@ -123,10 +196,13 @@ void create_month_texts()
       gtk_box_pack_start(GTK_BOX(glob_hbox_month_row[0]), hbox, TRUE, TRUE, 0);
       column++;
    }
-
+     
    for (n=0; n<ndim; n++) {
       glob_month_texts[n] = gtk_text_new(NULL, NULL);
       gtk_widget_set_usize(GTK_WIDGET(glob_month_texts[n]), 110, 90);
+      gtk_text_set_word_wrap(GTK_TEXT(glob_month_texts[n]), FALSE);
+      gtk_signal_connect(GTK_OBJECT(glob_month_texts[n]), "enter_notify_event",
+			 GTK_SIGNAL_FUNC(cb_enter_notify), GINT_TO_POINTER(n));
       gtk_box_pack_start(GTK_BOX(glob_hbox_month_row[row]), glob_month_texts[n], TRUE, TRUE, 0);
       if (++column > 6) {
 	 column=0;
@@ -152,40 +228,45 @@ void create_month_texts()
       gtk_widget_set_usize(GTK_WIDGET(hbox), 110, 90);
       gtk_box_pack_start(GTK_BOX(glob_hbox_month_row[row]), hbox, TRUE, TRUE, 0);
    }
-
 }
 
 int display_months_appts(struct tm *date_in, GtkWidget **day_texts)
 {
    AppointmentList *a_list;
    AppointmentList *temp_al;
-   long modified, deleted;
    struct tm date;
    GtkWidget **text;
-   char desc[60];
+   char desc[100];
    char datef[20];
    int dow;
    int ndim;
    int n;
-   GdkFont *small_font;
+#ifdef USE_DB3
+   int ret;
+   int category;
+   int cat_bit;
+   long use_db3_tags;
+#endif
 
    a_list = NULL;
    text = day_texts;
-
-   small_font = gdk_fontset_load("-misc-fixed-medium-r-*-*-*-100-*-*-*-*-*");
-
+/*
+   get_pref(PREF_CHAR_SET, &char_set, NULL);
+   if (char_set==CHAR_SET_1250) {
+       small_font = gdk_fontset_load("-misc-fixed-medium-r-*-*-*-100-*-*-*-iso8859-2");
+   } else {
+       small_font = gdk_fontset_load("-misc-fixed-medium-r-*-*-*-100-*-*-*-*-*");
+   }
+*/
    memcpy(&date, date_in, sizeof(struct tm));
 
    /* Get all of the appointments */
-   get_days_appointments(&a_list, NULL);
+   get_days_appointments2(&a_list, NULL, 2, 2, 2);
 
    get_month_info(date.tm_mon, 1, date.tm_year, &dow, &ndim);
 
    weed_datebook_list(&a_list, date.tm_mon, date.tm_year);
 
-   get_pref(PREF_SHOW_MODIFIED, &modified, NULL);
-   get_pref(PREF_SHOW_DELETED, &deleted, NULL);
-
    for (n=0, date.tm_mday=1; date.tm_mday<=ndim; date.tm_mday++, n++) {
       date.tm_sec=0;
       date.tm_min=0;
@@ -196,16 +277,18 @@ int display_months_appts(struct tm *date_in, GtkWidget **day_texts)
       mktime(&date);
 
       for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
-	 if (temp_al->ma.rt == MODIFIED_PALM_REC) {
-	    if (!modified) {
-	       continue;
-	    }
-	 }
-	 if (temp_al->ma.rt == DELETED_PALM_REC) {
-	    if (!deleted) {
+#ifdef USE_DB3
+	 get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
+	 if (use_db3_tags) {
+	    ret = db3_is_float(&(temp_al->ma.a), &category);
+	    jpilot_logf(LOG_DEBUG, "category = 0x%x\n", category);
+	    cat_bit=1<<category;
+	    if (!(cat_bit & datebook_category)) {
+	       jpilot_logf(LOG_DEBUG, "skipping rec not in this category\n");
 	       continue;
 	    }
 	 }
+#endif
 	 if (isApptOnDate(&(temp_al->ma.a), &date)) {
 	    if (temp_al->ma.a.event) {
 	       desc[0]='\0';
@@ -215,13 +298,15 @@ int display_months_appts(struct tm *date_in, GtkWidget **day_texts)
 	    }
 	    strcat(desc, " ");
 	    if (temp_al->ma.a.description) {
-	       strncat(desc, temp_al->ma.a.description, 20);
-	       desc[16]='\0';
+	       strncat(desc, temp_al->ma.a.description, 40);
+	       desc[35]='\0';
 	    }
 	    remove_cr_lfs(desc);
 	    strcat(desc, "\n");
+/*	    gtk_text_insert(GTK_TEXT(text[n]),
+			    small_font, NULL, NULL, desc, -1);*/
 	    gtk_text_insert(GTK_TEXT(text[n]),
-			    small_font, NULL, NULL, desc, -1);
+			    NULL, NULL, NULL, desc, -1);
 	 }
       }
    }
@@ -254,18 +339,25 @@ void monthview_gui(struct tm *date_in)
    char str[256];
    long fdow;
    const char *str_fdow;
+   char title[200];
 
    if (window) {
       return;
    }
    
+   for (i=0; i<6; i++) {
+      glob_hbox_month_row[i]=NULL;
+   }
+
    memcpy(&glob_month_date, date_in, sizeof(struct tm));
    
    get_pref(PREF_FDOW, &fdow, &str_fdow);
 
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width(GTK_CONTAINER(window), 10);
-   gtk_window_set_title(GTK_WINDOW(window), PN" Monthly View");
+   g_snprintf(title, 200, "%s %s", PN, _("Monthly View"));
+   title[199]='\0';
+   gtk_window_set_title(GTK_WINDOW(window), title);
 
    gtk_signal_connect(GTK_OBJECT(window), "destroy",
                       GTK_SIGNAL_FUNC(cb_destroy), window);
@@ -292,7 +384,7 @@ void monthview_gui(struct tm *date_in)
 
    
    /* Create a "Quit" button */
-   button = gtk_button_new_with_label("Close");
+   button = gtk_button_new_with_label(_("Close"));
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_quit), window);
    gtk_box_pack_start(GTK_BOX(hbox_temp), button, FALSE, FALSE, 0);
@@ -328,6 +420,10 @@ void monthview_gui(struct tm *date_in)
      
    create_month_texts();
 
+   big_text = gtk_text_new(NULL, NULL);
+   gtk_widget_set_usize(GTK_WIDGET(big_text), 0, 100);
+   gtk_box_pack_start(GTK_BOX(vbox), big_text, TRUE, TRUE, 0);
+
    display_months_appts(&glob_month_date, glob_month_texts);
  
    gtk_widget_show_all(window);
