@@ -4,8 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,7 +28,7 @@
  */
 int print_gui(GtkWidget *main_window);
 
-int print_datebook(int month, int day, int year);
+int print_datebook(struct tm *date, AppointmentList *a_list);
 
 int print_addresses(AddressList *address_list);
 
